// src/cli.ts
import { Command } from "commander";
import { existsSync, readFileSync } from "node:fs";

// src/aws_events/httpapi2.ts
var httpapi2_default = (queryPath) => ({
  version: "2.0",
  routeKey: "$default",
  rawPath: queryPath,
  rawQueryString: "",
  headers: {
    accept: "text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.7",
    //"*/*",
    "content-length": "0",
    host: "localhost",
    "user-agent": "PostmanRuntime/7.26.8",
    "x-amzn-trace-id": "Root=1-5f84c7a9-0e5b1e1e1e1e1e1e1e1e1e1e",
    "x-forwarded-for": "127.0.0.1",
    "x-forwarded-port": "443",
    "x-forwarded-proto": "https"
  },
  requestContext: {
    accountId: "123456789012",
    apiId: "api-id",
    domainName: "localhost",
    domainPrefix: "example",
    http: {
      method: "GET",
      path: queryPath,
      protocol: "HTTP/1.1",
      sourceIp: "127.0.0.1",
      userAgent: "PostmanRuntime/7.26.8"
    },
    requestId: "id",
    routeKey: "$default",
    stage: "$default",
    time: "12/Mar/2021:19:03:58 +0000",
    timeEpoch: 1615578238e3
  },
  isBase64Encoded: false
});

// src/library/framework.ts
function detectFramework(packageJson) {
  if (packageJson.dependencies) {
    if (packageJson.dependencies["waku"]) {
      return "waku";
    }
    if (packageJson.dependencies["vike"]) {
      return "vike";
    }
    if (packageJson.dependencies["@lazarv/react-server"]) {
      return "react-server";
    }
  }
  return void 0;
}
function getDefaultHandlerPath(framework) {
  switch (framework) {
    case "waku":
      return "dist/serve-aws-lambda.js";
    case "vike":
      return "entry_aws_lambda.ts";
    case "react-server":
      return ".aws-react-server/output/functions/index.func/index.mjs";
    default:
      return "handler.js";
  }
}

// src/cli.ts
import { resolve } from "node:path";

// src/library/ResponseStream.ts
import { Stream } from "stream";
var ResponseStream = class extends Stream.Writable {
  response;
  silent;
  _contentType;
  _isBase64Encoded;
  constructor({ silent } = { silent: false }) {
    super();
    this.response = Buffer.from("");
    this.silent = silent;
  }
  // @param chunk Chunk of data to unshift onto the read queue. For streams not operating in object mode, `chunk` must be a string, `Buffer`, `Uint8Array` or `null`. For object mode
  // streams, `chunk` may be any JavaScript value.
  _write(chunk, encoding, callback) {
    if (!this.silent) {
      const data = Buffer.from(chunk, encoding).toString("utf-8");
      process.stdout.write(data);
    }
    if (!this.response) {
      this.response = Buffer.from(chunk, encoding);
    } else {
      this.response = Buffer.concat([
        this.response,
        Buffer.from(chunk, encoding)
      ]);
    }
    callback();
  }
  getBufferedData() {
    return this.response;
  }
  setContentType(contentType) {
    this._contentType = contentType;
  }
  setIsBase64Encoded(isBase64Encoded) {
    this._isBase64Encoded = isBase64Encoded;
  }
};

// src/library/awslambda-simulator.ts
var awslambdaSimulator = (silent) => {
  let responseStream = new ResponseStream({ silent });
  return {
    awslambda: {
      streamifyResponse: (handler) => async (event, context) => {
        await handler(event, responseStream, context);
        return responseStream;
      },
      HttpResponseStream: {
        from(responseStream2, metadata) {
          return responseStream2;
        }
      }
    },
    streamifyHandler: (handler) => async (event, context) => {
      const responseStreamFinished = new Promise((resolve2, reject) => {
        responseStream.on("close", resolve2);
        responseStream.on("error", reject);
      });
      await handler(event, responseStream, context);
      await responseStreamFinished;
      return responseStream.getBufferedData().toString("utf-8");
    }
  };
};

// src/cli.ts
import chalk from "chalk";

// src/library/performanceObserver.ts
import { performance, PerformanceObserver } from "node:perf_hooks";
import EasyTable from "easy-table";
import Summary from "summary";
var Performance = class {
  isEnabled;
  startTime = 0;
  endTime = 0;
  entries = [];
  instanceId;
  fnObserver;
  constructor(isEnabled) {
    this.isEnabled = isEnabled;
    if (isEnabled) {
      this.startTime = performance.now();
      this.instanceId = Math.floor(performance.now() * 100);
      this.fnObserver = new PerformanceObserver((items) => {
        items.getEntries().forEach((entry) => {
          this.entries.push(entry);
        });
      });
      this.fnObserver.observe({ entryTypes: ["function"] });
    }
  }
  init() {
    this.startTime = performance.now();
    this.instanceId = Math.floor(performance.now() * 100);
    this.fnObserver = new PerformanceObserver((items) => {
      items.getEntries().forEach((entry) => {
        this.entries.push(entry);
      });
    });
    this.fnObserver.observe({ entryTypes: ["function"] });
  }
  timerify(fn) {
    return this.isEnabled ? performance.timerify(fn) : fn;
  }
  setMark(name) {
    const id = `${this.instanceId}:${name}`;
    performance.mark(`${id}:start`);
  }
  clearMark(name) {
    const id = `${this.instanceId}:${name}`;
    performance.mark(`${id}:end`);
    performance.measure(id, `${id}:start`, `${id}:end`);
    performance.clearMarks(`${id}:start`);
    performance.clearMarks(`${id}:end`);
  }
  async flush() {
    this.setMark("_flush");
    await new Promise((resolve2) => setTimeout(resolve2, 1));
    this.clearMark("_flush");
  }
  getEntriesByName() {
    return this.entries.reduce((entries, entry) => {
      const name = entry.name.replace(`${this.instanceId}:`, "");
      entries[name] = entries[name] ?? [];
      entries[name].push(entry.duration);
      return entries;
    }, {});
  }
  getTable() {
    const entriesByName = this.getEntriesByName();
    const table = new EasyTable();
    Object.entries(entriesByName).map(([name, values]) => {
      const stats = new Summary(values);
      table.cell("Name", name);
      table.cell("size", stats.size(), EasyTable.number(0));
      table.cell("min", stats.min(), EasyTable.number(2));
      table.cell("max", stats.max(), EasyTable.number(2));
      table.cell("median", stats.median(), EasyTable.number(2));
      table.cell("sum", stats.sum(), EasyTable.number(2));
      table.newRow();
    });
    table.sort(["sum|des"]);
    return table.toString().trim();
  }
  getTotalTime() {
    return this.endTime - this.startTime;
  }
  async finalize() {
    if (!this.isEnabled) return;
    this.endTime = performance.now();
    await this.flush();
  }
  reset() {
    this.entries = [];
    this.fnObserver?.disconnect();
  }
};

// src/cli.ts
import prettyMs from "pretty-ms";
import { fileURLToPath } from "url";
import { dirname } from "path";
(async () => {
  const __filename = fileURLToPath(import.meta.url);
  const __dirname = dirname(__filename);
  const toolPackageJsonPath = `${__dirname}/../package.json`;
  if (!existsSync(toolPackageJsonPath)) {
    console.error("Error: tool package.json not found!");
    process.exit(1);
  }
  const toolPackageJson = JSON.parse(
    readFileSync(toolPackageJsonPath, "utf-8")
  );
  const version = toolPackageJson.version;
  const packageJsonPath = `package.json`;
  const packageJson = existsSync(packageJsonPath) ? JSON.parse(readFileSync(packageJsonPath, "utf-8")) : { dependencies: {} };
  const program = new Command();
  const error = chalk.bold.red;
  const warning = chalk.hex("#FFA500");
  function myParseInt(value) {
    const parsedValue = parseInt(value, 10);
    if (isNaN(parsedValue)) {
      throw "Not a number.";
    }
    return parsedValue;
  }
  program.version(version).name("lambda-handler-tester").option("--handler [PATH]", "path to request handler").option("-s, --streaming", "streaming handler", false).option("-e, --event [PATH]", "path to an json file with a valid event").option("-c, --context [PATH]", "path to an json file with a valid context").option("-p, --path [PATH]", "path to request", "/").option("--decode-base64", "decode base64 body", false).option("--header", "only show header without body", false).option("-v, --verbose", "enables verbose logging", false).option("--response-time", "measure the response time", false).option("--repeat <number>", "repeat request [n] times", myParseInt, 1).option("--silent", "no output", false).option("-d, --debug", "enables verbose logging", false).parse(process.argv);
  const options = program.opts();
  let eventData = null;
  let contextData = null;
  let framework;
  let handlerPath = options.handler;
  let awsLambdaSimulator;
  if (handlerPath) {
    if (!existsSync(handlerPath)) {
      console.error(`Error: handler file not found: ${handlerPath}`);
      process.exit(1);
    }
  } else {
    framework = detectFramework(packageJson);
    if (framework === void 0) {
      console.error("Error: unknown framework! Please provide a handler path");
      process.exit(1);
    }
    handlerPath = getDefaultHandlerPath(framework);
  }
  const queryPath = options.path;
  if (options.event) {
    const eventPath = options.event;
    if (!existsSync(eventPath)) {
      console.error(`Error: event file not found: ${eventPath}`);
      process.exit(1);
    }
    try {
      eventData = JSON.parse(readFileSync(eventPath, { encoding: "utf-8" }));
    } catch (e) {
      console.error(`Error: invalid JSON: ${eventPath}`);
      process.exit(1);
    }
  } else {
    eventData = httpapi2_default(queryPath);
  }
  if (options.context) {
    const contextPath = options.context;
    if (!existsSync(contextPath)) {
      console.error(`Error: context file not found: ${contextPath}`);
      process.exit(1);
    }
    try {
      contextData = JSON.parse(
        readFileSync(contextPath, { encoding: "utf-8" })
      );
    } catch (e) {
      console.error(`Error: invalid JSON: ${contextPath}`);
      process.exit(1);
    }
  }
  if (options.debug) {
    console.log("Options:", options);
    console.log("handlerPath:", handlerPath);
    console.log("streaming:", options.streaming);
  }
  if (options.verbose) {
    console.log("Event:", eventData);
    console.log("Context:", contextData);
  }
  if (options.streaming) {
    awsLambdaSimulator = awslambdaSimulator(
      options.silent || options.responseTime
    );
    globalThis.awslambda = awsLambdaSimulator.awslambda;
  }
  const perfObserver = new Performance(options.responseTime);
  try {
    const { handler: handlerImported } = await import(resolve(handlerPath));
    const handler = options.streaming ? awsLambdaSimulator?.streamifyHandler(handlerImported) : handlerImported;
    const handlerRepeat = options.streaming ? awsLambdaSimulator?.streamifyHandler(handlerImported) : handlerImported;
    const handlerWrapped = perfObserver.timerify(handler);
    let responseData;
    responseData = await handlerWrapped(eventData, contextData);
    if (options.responseTime) {
      await perfObserver.finalize();
      console.log(`
First run:`);
      console.log(perfObserver.getTable());
      perfObserver.reset();
      perfObserver.init();
    }
    if (options.repeat > 1) {
      process.stdout.write(`
Repeat ${options.repeat - 1} times: `);
      for (let i = 0; i < options.repeat; i++) {
        const progress = Math.floor(i / options.repeat * 100);
        if (progress % 10 === 0) {
          process.stdout.write(` ${String(progress).padStart(3, " ")}%`);
          process.stdout.write("\x1B[5D");
        }
        responseData = await handlerWrapped(eventData, contextData);
      }
    }
    process.stdout.write(` 100%
`);
    if (options.responseTime) {
      await perfObserver.finalize();
      console.log(perfObserver.getTable());
      console.log("Total running time:", prettyMs(perfObserver.getTotalTime()));
    }
    if (options.silent || options.responseTime) process.exit(0);
    const isBase64Encoded = responseData?.isBase64Encoded === true;
    if (options.header) {
      if (responseData.body) delete responseData.body;
      console.log(responseData);
      process.exit(0);
    }
    if (isBase64Encoded) {
      responseData.isBase64Encoded = false;
      responseData.body = Buffer.from(responseData.body, "base64").toString();
    }
    if (options.verbose) {
      console.log(responseData);
      if (isBase64Encoded) {
        console.log(
          warning(
            chalk.bold("\u26A0\uFE0F Note:") + "The original response was base64 encoded!"
          )
        );
      }
    } else {
      console.log(responseData?.body ?? "");
    }
    process.exit(0);
  } catch (e) {
    console.error(e);
    process.exit(1);
  }
})();
//# sourceMappingURL=cli.js.map